Package: MatchIt
Version: 4.7.2
Title: Nonparametric Preprocessing for Parametric Causal Inference
Description: Selects matched samples of the original treated and
    control groups with similar covariate distributions -- can be
    used to match exactly on covariates, to match on propensity
    scores, or perform a variety of other matching procedures.  The
    package also implements a series of recommendations offered in
    Ho, Imai, King, and Stuart (2007) <DOI:10.1093/pan/mpl013>. (The 
    'gurobi' package, which is not on CRAN, is optional and comes with 
    an installation of the Gurobi Optimizer, available at 
    <https://www.gurobi.com>.)
Authors@R: c(
  person("Daniel", "Ho", email = "daniel.e.ho@gmail.com", role = c("aut"),
         comment = c(ORCID = "0000-0002-2195-5469")),
  person("Kosuke", "Imai", email = "imai@harvard.edu", role = c("aut"),
         comment = c(ORCID = "0000-0002-2748-1022")),
  person("Gary", "King", email = "king@harvard.edu", role = c("aut"),
         comment = c(ORCID = "0000-0002-5327-7631")),
  person("Elizabeth", "Stuart", email = "estuart@jhu.edu", role = c("aut"),
         comment = c(ORCID = "0000-0002-9042-8611")),
  person("Alex", "Whitworth", email = "whitworth.alex@gmail.com", role = c("ctb")),
  person("Noah", "Greifer", role = c("cre", "aut"), 
         email = "noah.greifer@gmail.com",
	       comment = c(ORCID="0000-0003-3067-7154"))
  )
Depends: R (>= 3.6.0)
Imports: backports (>= 1.1.9), chk (>= 0.10.0), rlang (>= 1.1.0), Rcpp,
        utils, stats, graphics, grDevices
Suggests: optmatch (>= 0.10.6), Matching, rgenoud, quickmatch (>=
        0.2.1), nnet, rpart, mgcv, CBPS (>= 0.17), dbarts (>= 0.9-28),
        randomForest (>= 4.7-1), glmnet (>= 4.0), gbm (>= 2.1.7),
        cobalt (>= 4.2.3), boot, marginaleffects (>= 0.25.0), sandwich
        (>= 2.5-1), survival, RcppProgress (>= 0.4.2), highs, Rglpk,
        Rsymphony, gurobi, knitr, rmarkdown, testthat (>= 3.0.0)
LinkingTo: Rcpp, RcppProgress
Encoding: UTF-8
LazyData: true
License: GPL (>= 2)
URL: https://kosukeimai.github.io/MatchIt/,
        https://github.com/kosukeimai/MatchIt
BugReports: https://github.com/kosukeimai/MatchIt/issues
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: yes
Packaged: 2025-05-29 23:12:28 UTC; NoahGreifer
Author: Daniel Ho [aut] (ORCID: <https://orcid.org/0000-0002-2195-5469>),
  Kosuke Imai [aut] (ORCID: <https://orcid.org/0000-0002-2748-1022>),
  Gary King [aut] (ORCID: <https://orcid.org/0000-0002-5327-7631>),
  Elizabeth Stuart [aut] (ORCID: <https://orcid.org/0000-0002-9042-8611>),
  Alex Whitworth [ctb],
  Noah Greifer [cre, aut] (ORCID:
    <https://orcid.org/0000-0003-3067-7154>)
Maintainer: Noah Greifer <noah.greifer@gmail.com>
Repository: CRAN
Date/Publication: 2025-05-30 09:00:09 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 07:56:20 UTC; unix
Archs: MatchIt.so.dSYM
