# Generated by roxygen2: do not edit by hand

S3method(plot,matchit)
S3method(plot,matchit.subclass)
S3method(plot,summary.matchit)
S3method(print,matchit)
S3method(print,summary.matchit)
S3method(print,summary.matchit.subclass)
S3method(rbind,getmatches)
S3method(rbind,matchdata)
S3method(summary,matchit)
S3method(summary,matchit.subclass)
export(add_s.weights)
export(euclidean_dist)
export(get_matches)
export(mahalanobis_dist)
export(match.data)
export(match_data)
export(matchit)
export(robust_mahalanobis_dist)
export(scaled_euclidean_dist)
import(graphics)
import(stats)
importFrom(Rcpp,evalCpp)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,nclass.FD)
importFrom(grDevices,nclass.Sturges)
importFrom(grDevices,nclass.scott)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,hasName)
useDynLib(MatchIt, .registration = TRUE)
