#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import graphics
#' @import stats
#' @importFrom grDevices devAskNewPage
#' @importFrom grDevices nclass.FD
#' @importFrom grDevices nclass.scott
#' @importFrom grDevices nclass.Sturges
#' @importFrom Rcpp evalCpp
#' @importFrom utils capture.output
#' @importFrom utils combn
#' @importFrom utils hasName
#' @useDynLib MatchIt, .registration = TRUE
## usethis namespace: end
NULL
