# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

all_equal_to <- function(x, y) {
    .Call(`_MatchIt_all_equal_to`, x, y)
}

eucdistC_N1xN0 <- function(x, t) {
    .Call(`_MatchIt_eucdistC_N1xN0`, x, t)
}

get_splitsC <- function(x, caliper) {
    .Call(`_MatchIt_get_splitsC`, x, caliper)
}

has_n_unique <- function(x, n) {
    .Call(`_MatchIt_has_n_unique`, x, n)
}

nn_matchC_distmat <- function(treat_, ord, ratio, discarded, reuse_max, focal_, distance_mat, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_distmat`, treat_, ord, ratio, discarded, reuse_max, focal_, distance_mat, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, disl_prog)
}

nn_matchC_distmat_closest <- function(treat, ratio, discarded, reuse_max, distance_mat, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, close = TRUE, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_distmat_closest`, treat, ratio, discarded, reuse_max, distance_mat, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, close, disl_prog)
}

nn_matchC_mahcovs <- function(treat_, ord, ratio, discarded, reuse_max, focal_, mah_covs, distance_ = NULL, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_mahcovs`, treat_, ord, ratio, discarded, reuse_max, focal_, mah_covs, distance_, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, disl_prog)
}

nn_matchC_mahcovs_closest <- function(treat, ratio, discarded, reuse_max, mah_covs, distance_ = NULL, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, close = TRUE, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_mahcovs_closest`, treat, ratio, discarded, reuse_max, mah_covs, distance_, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, close, disl_prog)
}

nn_matchC_vec <- function(treat_, ord, ratio, discarded, reuse_max, focal_, distance, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_vec`, treat_, ord, ratio, discarded, reuse_max, focal_, distance, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, disl_prog)
}

nn_matchC_vec_closest <- function(treat, ratio, discarded, reuse_max, distance, exact_ = NULL, caliper_dist_ = NULL, caliper_covs_ = NULL, caliper_covs_mat_ = NULL, antiexact_covs_ = NULL, unit_id_ = NULL, close = TRUE, disl_prog = FALSE) {
    .Call(`_MatchIt_nn_matchC_vec_closest`, treat, ratio, discarded, reuse_max, distance, exact_, caliper_dist_, caliper_covs_, caliper_covs_mat_, antiexact_covs_, unit_id_, close, disl_prog)
}

pairdistsubC <- function(x, t, s) {
    .Call(`_MatchIt_pairdistsubC`, x, t, s)
}

preprocess_matchC <- function(t, p) {
    .Call(`_MatchIt_preprocess_matchC`, t, p)
}

subclass2mmC <- function(subclass_, treat, focal) {
    .Call(`_MatchIt_subclass2mmC`, subclass_, treat, focal)
}

mm2subclassC <- function(mm, treat, focal = NULL) {
    .Call(`_MatchIt_mm2subclassC`, mm, treat, focal)
}

subclass_scootC <- function(subclass_, treat_, x_, min_n) {
    .Call(`_MatchIt_subclass_scootC`, subclass_, treat_, x_, min_n)
}

tabulateC <- function(bins, nbins = NULL) {
    .Call(`_MatchIt_tabulateC`, bins, nbins)
}

weights_matrixC <- function(mm, treat_, focal = NULL) {
    .Call(`_MatchIt_weights_matrixC`, mm, treat_, focal)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_MatchIt_RcppExport_registerCCallable`)
})
